
/*
	Hello World example

	This example shows how to represent the classic
	'helloworld.c' program using coast classes.

	You need to link coast.o object to compile this
	source.

	(c) 2003 Gabriele Budelacci.
	This file is released under the terms of GPL.
*/


#include <stdlib.h>
#include <stdio.h>

#include "coast.hpp"
#include "c_code_provider.hpp"


/*
	The source file to be represented is:


		#include <stdio.h>

		int main( int argc, char* argv[] )
		{
			printf( "Hello World!!!\n" );
			return 0;
		}

*/


int main( int argc, char* argv[] )
{
	/*
		FIRST PASS:
		We need to represent the source code.
	*/

	// define an application
    CoastApplication* application = new CoastApplication();

	// add a snippet member to application
	application->members->add( new CodeSnippetMember( "#include <stdio.h>" ) );

		// define the main entry point
	    CoastEntryPointFunction* entrypoint = new CoastEntryPointFunction();

			// define a 'printf' function invoke expression
			CoastFunctionInvokeExpression* printf_invoke = new CoastFunctionInvokeExpression( "printf" );

			// add the 'hello world' string as first invoking argument
			//	(note the double backslash '\\n' to represent the newline '\n')
			printf_invoke->arguments->add( new CoastPrimitiveExpression( "Hello World!!!\\n" ) );

		// add a expression statement to entry point statements
		entrypoint->statements->add( new CoastExpressionStatement( printf_invoke ) );

		// define a return statement
		CoastReturnStatement ret_stat = new CoastReturnStatement( new CoastPrimitiveExpression( 0 ) );

		// add the return statement to entry point
		entrypoint->statements->add( ret_stat );

	// add the entry point to application members
    application->members->add( entrypoint );



	/*
		SECOND PASS:
		Now we can operate over the source code.
	*/

	// create a C code provider
	CCodeProvider* provider = new CCodeProvider();

	// get the assembly source of the application
	string assembly = provider->generate_code( application );



	// output the application source
	fprintf( stdout, assembly.c_str() );

    return 0;

}//main

